import java.util.ArrayList;



public class VHDL_Tokenizer 
{
	private String line = null;
	int index = 0;
	private boolean nextCharValid = false;
	private char nextChar = (char)-2;
	
	public VHDL_Tokenizer(String line)
	{
		this.line = line;
	}
	
	public ArrayList <String> getTokens()
	{
		ArrayList <String> rv = new ArrayList <String>();
		
		for (String s = nextToken(); s != null; s = nextToken())
			rv.add(s);
		
		return rv;
	}
	
	public String nextToken()
	{
		String rv = "";
		
		for (char l = nextChar(); l != (char)-1; l = nextChar())
		{
			l = Character.toLowerCase(l);
			
			if (rv.length() > 0) // check if we should end the current token
			{
				char enders[] = {';', ':', '(', ')', '+', '/', '*', '.'};
				boolean end = false;
				
				if (Character.isWhitespace(l))
					end = true;
				
				for (int x = 0; x < enders.length; ++x)
				{
					if (l == enders[x])
					{
						end = true;
						putBack(l);
						break;
					}
				}
				
				// see if the token consist of a forced complete token
				String completeTokens[] = {":=", ";", "(", ")", "+", "/", "*",
							"<=", ">=", "."};
				
				for (int x = 0; x < completeTokens.length; ++x)
				{
					if (rv.equals(completeTokens[x]))
					{
						end = true;
						putBack(l);
						break;
					}
				}
				
				// see if a non-space delimited token has just completed
				if ((rv.equals("<") && l != '=') || (rv.equals(">") && l != '=') ||
						(rv.equals(":") && l != '=') || (rv.equals("-") && l != '-')
						|| (!rv.equals("-") && l == '-'))
				{
					end = true;
					putBack(l);
				}
				
				if (end)
					break; // exit out of token character-adding loop
			}
			
			// skip whitespace
			if (!Character.isWhitespace(l))
			{
				// append letter to current token
				rv += l;
				
				if (rv.equals("--")) // current token is a comment
				{
					// clear current token
					rv = "";
					
					// skip to the end of the line
					index = line.length();
				}
			}
		}			
		
		
		if (rv.length() == 0)
			rv = null;
		
		return rv;
	}	
	
	/**
	 * Read the next character from the stream
	 * @return the next character from the stream, or -1 on EOF/error
	 */
	private char nextChar()
	{
		char rv = (char)-1;
		
		if (nextCharValid)
		{
			nextCharValid = false;
			rv = nextChar;
			nextChar = (char)-2;
		}
		else
		{		
			if (index < line.length())
			{
				rv = line.charAt(index++);
			}
		}
		
		return rv;
	}
	
	/**
	 * Put a character back into the stream so it's the first one read by nextChar
	 */
	private void putBack(char c)
	{
		nextChar = c;
		nextCharValid = true;
	}
}
